<?php

/**
 * WooCommerce B2B Sales Agents Products
 *
 * @version 1.2.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Admin_Products
 */
class WCB2BSA_Admin_Products {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'woocommerce_product_data_tabs', array( $this, 'add_tab' ) );
        add_action( 'woocommerce_product_data_panels', array( $this, 'add_tab_panel' ) );
        add_action( 'woocommerce_process_product_meta', array( $this, 'save_tab_values' ) );
        add_filter( 'woocommerce_product_export_column_names', array( $this, 'add_export_column' ) );
        add_filter( 'woocommerce_product_export_product_default_columns', array( $this, 'add_export_column' ) );
        add_filter( 'woocommerce_product_export_product_query_args', array( $this, 'query_args' ) );
        add_filter( 'woocommerce_csv_product_import_mapping_options', array( $this, 'import_mapping_options' ) );
        add_filter( 'woocommerce_product_import_pre_insert_product_object', array( $this, 'import_pre_insert_product_object' ), 10, 2 );
        add_filter( 'woocommerce_csv_product_import_mapping_default_columns', array( $this, 'add_column_to_mapping_screen' ) );

        if ( get_option( 'wcb2bsa_edit_prices' ) === 'yes' ) {
            add_action( 'woocommerce_product_options_pricing', array( $this, 'add_product_price_hook' ) );
            add_action( 'woocommerce_variation_options_pricing', array( $this, 'add_variation_price_hook' ), 10, 3 );
            add_action( 'woocommerce_process_product_meta', array( $this, 'save_product_group_prices' ) );
            add_action( 'woocommerce_save_product_variation', array( $this, 'save_product_group_prices' ) );
        }
    }

    /**
     * Add new field to manage group product prices in WooCommerce products
     */
    public function add_product_price_hook() {
        global $post;
        $this->build_product_price_fields( $post->ID, false );
    }

    /**
     * Add new field to manage group product prices in WooCommerce variations
     */
    public function add_variation_price_hook( $loop, $variation_data, $variation ) {
        $this->build_product_price_fields( $variation->ID, true );
    }

    /**
     * Build group prices fields
     *
     * @param int $post_id Current product ID
     * @param bool $is_variation Return if current product is a variation
     */
    public function build_product_price_fields( $post_id, $is_variation ) {
        // Retrieve prices and groups
        $product_min_price = get_post_meta( $post_id, 'wcb2bsa_min_price', true );

        include WCB2BSA_ABSPATH . 'includes/views/html-admin-product-prices.php';
    }

    /**
     * Save value into database
     * 
     * @param int $product_id Current product ID
     */
    public function save_product_group_prices( $product_id ) {
        $product_min_price = isset( $_POST['wcb2bsa_min_price'][$product_id] ) ? $_POST['wcb2bsa_min_price'][$product_id] : 0;
        update_post_meta( $product_id, 'wcb2bsa_min_price', wc_format_decimal( $product_min_price, '' ) );
    }

    /**
     * Add the new product tab (only on simple, variable, grouped)
     * 
     * @param array $tabs Contains tabs data
     * @return array
     */
    public function add_tab( $tabs ) {
        $tabs['wcb2bsa_tab'] = array(
            'label'     => __( 'Commissions', 'woocommerce-b2b-sales-agents' ),
            'target'    => 'wcb2bsa_product_data',
            'class'     => array( 'show_if_simple', 'show_if_variable', 'hide_if_external' ),
            'priority'  => 90
        );
        return $tabs;
    }

    /**
     * Add new field to manage sales agents commission in WooCommerce products
     */
    public function add_tab_panel() {
        global $post;

        $sales_agents = wcb2bsa_get_sales_agents();
        $commissions = get_post_meta( $post->ID, 'wcb2bsa_commissions', true );

        include_once WCB2BSA_ABSPATH . 'includes/views/html-admin-product-commissions.php';
    }

    /**
     * Save value into database
     *
     * @param int $product_id Product ID
     */
    public function save_tab_values( $product_id ) {
        if ( isset( $_POST['wcb2bsa_commissions'] ) ) {
            // Validate field
            update_post_meta( $product_id, 'wcb2bsa_commissions', $_POST['wcb2bsa_commissions'] );
        }
    }

    /**
     * Add commissions fields to export tools
     * 
     * @param array $columns Columns definition
     */
    public function add_export_column( $columns ) {
        // Retrieve all sales agents
        $sales_agents = wcb2bsa_get_sales_agents();

        $columns['wcb2bsa_min_price'] = __( 'Sales agents minimum price', 'woocommerce-b2b-sales-agents' );
        foreach ( $sales_agents->get_results() as $sales_agent ) {
            $columns['wcb2bsa_commissions_' . $sales_agent->ID] = sprintf( __( '%s (%%)', 'woocommerce-b2b-sales-agents' ), $sales_agent->display_name );
            $columns['wcb2bsa_commissions_' . $sales_agent->ID] = sprintf( __( '%s (%%)', 'woocommerce-b2b-sales-agents' ), $sales_agent->display_name );
        }

        return $columns;
    }

    /**
     * Manage commissions fields in export tools
     * 
     * @param  array $args Arguments for WP_Query
     * @return array
     */
    public function query_args( $args ) {
        // Retrieve all sales agents
        $sales_agents = wcb2bsa_get_sales_agents();

        foreach ( $sales_agents->get_results() as $sales_agent ) {
            // Provide the data to be exported for one item in the column
            add_filter( 'woocommerce_product_export_product_column_wcb2bsa_commissions_' . $sales_agent->ID, function( $value, $product, $column_id ) {
                $sales_agent_id = (int) ltrim( filter_var( $column_id, FILTER_SANITIZE_NUMBER_INT ), '2' ); 
                $commissions = $product->get_meta( 'wcb2bsa_commissions' );

                if ( is_array( $commissions ) && array_key_exists( $sales_agent_id, $commissions ) ) {
                    return $commissions[$sales_agent_id];
                }

                return $value;
            }, 10, 3 );
        }

        // Min price
        add_filter( 'woocommerce_product_export_product_column_wcb2bsa_min_price', function( $value, $product, $column_id ) {
            return number_format(
                floatval( $product->get_meta( 'wcb2bsa_min_price' ) ),
                wc_get_price_decimals(),
                wc_get_price_decimal_separator(),
                wc_get_price_thousand_separator()
            );
        }, 10, 3 );

        return $args;
    }

    /**
     * Register the columns in the importer
     * 
     * @param  array $options Mapping options
     * @return array
     */
    public function import_mapping_options( $options ) {
        // Retrieve all sales agents
        $sales_agents = wcb2bsa_get_sales_agents();
        $values = array();
        foreach ( $sales_agents->get_results() as $sales_agent ) {
            $values['wcb2bsa_commissions:' . $sales_agent->ID] = sprintf( __( 'Commission for %s (%%)', 'woocommerce-b2b-sales-agents' ), $sales_agent->display_name );
        }

        // Add commissions option to columns
        $options['wcb2bsa_commissions'] = array(
            'name' => __( 'WCB2BSA - Product commissions', 'woocommerce-b2b-sales-agents' ),
            'options' => $values
        );

        // Min price
        $options['wcb2bsa_min_price'] = array(
            'name' => __( 'WCB2BSA - Sales agents minimum price' ),
            'options' => array(
                'wcb2bsa_min_price' => __( 'Sales agents minimum price', 'woocommerce-b2b-sales-agents' )
            )
        );

        return $options;
    }

    /**
     * Process the data read from the CSV file
     * 
     * @return object
     */
    public function import_pre_insert_product_object( $object, $data ) {
        // Retrieve all groups and prices
        $commissions = $object->get_meta( 'wcb2bsa_commissions', true );
        if ( ! $commissions ) {
            $commissions = array();
        }
        $sales_agents = wcb2bsa_get_sales_agents();

        foreach ( $sales_agents->get_results() as $sales_agent ) {
            // Set the new commission (if exists)
            if ( isset( $data['wcb2bsa_commissions:' . $sales_agent->ID] ) ) {
                $value = $data['wcb2bsa_commissions:' . $sales_agent->ID];
                if ('' != $value) {
                    $value = wc_format_decimal( $value, '' );
                }
                $commissions[$sales_agent->ID] = $value;
            }
            
        }
        // Save the group prices
        $object->update_meta_data( 'wcb2bsa_commissions', $commissions );

        // Save min price
        if ( isset( $data['wcb2bsa_min_price'] ) ) {
            $min_price = $data['wcb2bsa_min_price'];
            $object->update_meta_data( 'wcb2bsa_min_price', wc_format_decimal( $min_price, '' ) );
        }

        return $object;
    }

    /**
     * Map columns
     */
    public function add_column_to_mapping_screen( $columns ) {
        // Retrieve all sales agents
        $sales_agents = wcb2bsa_get_sales_agents();
        foreach ( $sales_agents->get_results() as $sales_agent ) {
            $commission = sprintf( __( '%s (%%)', 'woocommerce-b2b-sales-agents' ), $sales_agent->display_name );
            $columns[$commission] = 'wcb2bsa_commissions:' . $sales_agent->ID;
        }

        $min_price = __( 'Sales agents minimum price', 'woocommerce-b2b-sales-agents' );
        $columns[$min_price] = 'wcb2bsa_min_price';
        return $columns;
    }

}

return new WCB2BSA_Admin_Products();