<?php

/**
 * WooCommerce B2B Sales Agents Admin Users set-up Class
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Admin_Users Class
 */
class WCB2BSA_Admin_Users {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'get_the_author_wcb2bsa_affiliate_id', array( $this, 'generate_affiliate_id' ), 10, 3 );
        add_action( 'show_user_profile', array( $this, 'user_profile' ) );
        add_action( 'edit_user_profile', array( $this, 'user_profile' ) );
        add_action( 'user_profile_update_errors', array( $this, 'validate_profile_fields' ), 0, 3 );
        add_action( 'personal_options_update', array( $this, 'user_profile_update' ) );
        add_action( 'edit_user_profile_update', array( $this, 'user_profile_update' ) );
        add_filter( 'manage_users_columns', array( $this, 'render_columns' ) );
        add_filter( 'manage_users_custom_column', array( $this, 'render_columns_value' ), 11, 3 );
        add_filter( 'manage_users_sortable_columns', array( $this, 'make_columns_sortable' ) );
        add_filter( 'bulk_actions-users', array( $this, 'bulk_actions' ) );
        add_filter( 'handle_bulk_actions-users', array( $this, 'bulk_actions_handle' ), 10, 3 );
        add_filter( 'pre_get_users', array( $this, 'query_filters' ) );
        add_action( 'manage_users_extra_tablenav', array( $this, 'render_filters' ) );
        add_action( 'woocommerce_coupon_options_usage_restriction', array( $this, 'coupon_restriction' ), 10, 2 );
        add_action( 'woocommerce_coupon_options_save', array( $this, 'coupon_options_save' ), 10, 2 );
        add_filter( 'woocommerce_shop_manager_editable_roles', array( $this, 'shop_manager_editable_roles' ) );
    }

    /**
     * Generate a unique affiliate ID
     *
     * @param string $value The value of the metadata
     * @param integer $user_id The user ID for the value
     * @param int|false $original_user_id The original user ID, as passed to the function
     * @return string
     */
    public function generate_affiliate_id( $value, $user_id, $original_user_id ) {
        if ( wcb2bsa_has_role( $original_user_id, 'sales_agent' ) ) {
            if ( empty( $value ) ) {
                $value = wcb2bsa_generate_affiliate_id( $original_user_id );
            }
        }
        return $value;
    }

    /**
     * Add sales agents field to user edit page
     * 
     * @param WP_User $user The current WP_User object
     */
    public function user_profile( $user ) {
        include_once WCB2BSA_ABSPATH . 'includes/views/html-admin-user.php';
    }

    /**
     * Validate affiliate ID to check is unique
     * 
     * @param object &$errors WP_Error object
     * @param boolean $update Whether this is a user update
     * @param object &$user User object
     */
    public function validate_profile_fields( &$errors, $update, &$user ) {
        if ( wcb2bsa_has_role( $user->ID, 'sales_agent' ) ) {
            if ( isset( $_POST['wcb2bsa_affiliate_id'] ) && !empty( $_POST['wcb2bsa_affiliate_id'] ) ) {
                $affiliate_id = sanitize_text_field( $_POST['wcb2bsa_affiliate_id'] );
                $users = get_users( array(
                    'exclude' => array( $user->ID ),
                    'meta_key' => 'wcb2bsa_affiliate_id',
                    'meta_value' => $affiliate_id
                ) );
                if ( count( $users ) > 0 ) {
                    return $errors->add( 'wcb2bsa_affiliate_id', __( 'Affiliate ID already exists, please choose a new unique one.', 'woocommerce-b2b-sales-agents' ) );
                }
            } else {
                return $errors->add( 'wcb2bsa_affiliate_id', __( 'Affiliate ID field is required.', 'woocommerce-b2b-sales-agents' ) );
            }

            // Affiliate ID
            $affiliate_id = sanitize_text_field( $_POST['wcb2bsa_affiliate_id'] );
            update_user_meta( $user->ID, 'wcb2bsa_affiliate_id', $affiliate_id );
        }
    }

    /**
     * Update sales agents field in user edit page
     * 
     * @param int $user_id The user ID
     */
    public function user_profile_update( $user_id ) {
        if ( wcb2bsa_has_role( $user_id, 'customer' ) ) {
            update_user_meta( $user_id, 'wcb2bsa_sales_agent', isset( $_POST['wcb2bsa_sales_agent'] ) ? $_POST['wcb2bsa_sales_agent'] : '' );
        }
        if ( wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
            if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
                $current_status = (int)get_the_author_meta( 'wcb2bsa_status', $user_id );
                update_user_meta( $user_id, 'wcb2bsa_status', intval( $_POST['wcb2bsa_status'] ) );

                // If status is enabled, send approval confirmation to customer
                if ( intval( $_POST['wcb2bsa_status'] ) && apply_filters( 'wcb2bsa_send_activation_notification', true ) ) {
                    if ( $current_status == 0 ) {
                        $emails = WC_Emails::instance()->emails;
                        if ( ! array_key_exists( 'WCB2BSA_Email_Sales_Agent_Status_Notification', $emails ) ) {
                            $emails['WCB2BSA_Email_Sales_Agent_Status_Notification'] = include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-email-sales_agent-status-notification.php';
                        }
                        do_action( 'wcb2bsa_sales_agent_status_notification', $user_id );
                    }
                }
                // If status is disabled, logout from everywhere
                if ( 0 == intval( $_POST['wcb2bsa_status'] ) ) {
                    $sessions = WP_Session_Tokens::get_instance( $user_id );
                    $sessions->destroy_all();
                }
            }

            // Global commission
            update_user_meta( $user_id, 'wcb2bsa_commission', isset( $_POST['wcb2bsa_commission'] ) ? $_POST['wcb2bsa_commission'] : 0 );
            // Global subcommission
            update_user_meta( $user_id, 'wcb2bsa_subcommission', isset( $_POST['wcb2bsa_subcommission'] ) ? $_POST['wcb2bsa_subcommission'] : 0 );
            // Order notification
            update_user_meta( $user_id, 'wcb2bsa_order_notification', isset( $_POST['wcb2bsa_order_notification'] ) ? $_POST['wcb2bsa_order_notification'] : 0 );
            update_user_meta( $user_id, 'wcb2bsa_payment_method', isset( $_POST['wcb2bsa_payment_method'] ) ? sanitize_text_field( $_POST['wcb2bsa_payment_method'] ) : '' );
            update_user_meta( $user_id, 'wcb2bsa_payment_method_info', isset( $_POST['wcb2bsa_payment_method_info'] ) ? sanitize_textarea_field( $_POST['wcb2bsa_payment_method_info'] ) : '' );
            // Parent sales agent
            if ( isset( $_POST['wcb2bsa_sales_agent_parent'] ) ) {
                update_user_meta( $user_id, 'wcb2bsa_sales_agent_parent', $_POST['wcb2bsa_sales_agent_parent'] );
            }
            // Default group (WooCommerce B2B compatibility)
            if ( isset( $_POST['wcb2bsa_customers_default_group'] ) ) {
                update_user_meta( $user_id, 'wcb2bsa_customers_default_group', $_POST['wcb2bsa_customers_default_group'] );
            }
        }
    }

    /**
     * Add new customers columns in users list
     * 
     * @param  array $columns Array of column name => label
     * @return array
     */
    public function render_columns( $columns ) {
        // Display customer sales agent
        if ( ! isset( $_GET['role'] ) || ( isset( $_GET['role'] ) && get_option( 'wcb2bsa_has_role_customer', array( 'customer' ) ) ) ) {
            foreach ( $columns as $column_name => $column_info ) {
                if ( 'posts' === $column_name ) {
                    $_columns['wcb2bsa_sales_agent'] = esc_html__( 'Sales agent', 'woocommerce-b2b-sales-agents' );
                }
                $_columns[ $column_name ] = $column_info;
            }
            return $_columns;
        }
        // Display sales agent parent
        if ( ! isset( $_GET['role'] ) || ( isset( $_GET['role'] ) && in_array( $_GET['role'], get_option( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ) ) ) ) {
            foreach ( $columns as $column_name => $column_info ) {
                if ( 'posts' === $column_name ) {
                    $_columns['wcb2bsa_sales_agent_parent'] = esc_html__( 'Manager', 'woocommerce-b2b-sales-agents' );
                    if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
                        $_columns['wcb2bsa_status'] = esc_html__( 'Status', 'woocommerce-b2b-sales-agents' );
                    }
                }
                $_columns[ $column_name ] = $column_info;
            }
            return $_columns;
        }

        return $columns;
    }

    /**
     * Retrieve customers column value in users list
     *
     * @param string $value Custom column output
     * @param string $column_name Column name
     * @param int $user_id ID of the currently-listed user
     * @return string
     */
    public function render_columns_value( $value, $column_name, $user_id ) {
        if ( wcb2bsa_has_role( $user_id, 'customer' ) ) {
            if ( $column_name === 'wcb2bsa_sales_agent' ) {
                if ( $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $user_id ) ) {
                    $sales_agent = get_userdata( $sales_agent_id );
                    if ( false !== $sales_agent && wcb2bsa_has_role( $user_id, 'customer' ) ) {
                        return $sales_agent->display_name;
                    }
                }
            }
        }
        if ( wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
            if ( $column_name === 'wcb2bsa_sales_agent_parent' ) {
                if ( $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent_parent', $user_id ) ) {
                    $sales_agent = get_userdata( $sales_agent_id );
                    if ( false !== $sales_agent && wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
                        return $sales_agent->display_name;
                    }
                }
            }
            if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
                if ( $column_name === 'wcb2bsa_status' ) {
                    return get_the_author_meta( 'wcb2bsa_status', $user_id ) ? '<span class="dashicons dashicons-yes"></span>' : '<span class="dashicons dashicons-no-alt"></span>';
                }
            }
        }
        return $value;
    }

    /**
     * Make sales agents column sortable in users list
     * 
     * @param  array $columns Array of column name => label
     * @return array
     */
    public function make_columns_sortable( $columns ) {
        $columns['wcb2bsa_sales_agent'] = 'wcb2bsa_sales_agent';
        $columns['wcb2bsa_sales_agent_parent'] = 'wcb2bsa_sales_agent_parent';
        if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
            $columns['wcb2bsa_status'] = 'wcb2bsa_status';
        }
        return $columns;
    }

    /**
     * Bulk assignment to sales agent
     * 
     * @param  array $actions Array of the available bulk actions
     * @return array
     */
    public function bulk_actions( $actions ) {
        $actions['wcb2bsa-assign_sales_agents-action'] = esc_html__( 'Assign sales agent', 'woocommerce-b2b-sales-agents' );
        if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
            $actions['wcb2bsa-change_status-action'] = __( 'Change sales agent status', 'woocommerce-b2b-sales-agents' );
        }
        return $actions;
    }

    /**
     * Process new sales agent assignment and return updated users
     * 
     * @param string $redirect_url The redirect URL
     * @param string $action The action being taken
     * @param array $items The items to take the action on
     * @return string
     */
    public function bulk_actions_handle( $redirect_url, $action, $items ) {
        if ( $action == 'wcb2bsa-assign_sales_agents-action' && isset( $_REQUEST['wcb2bsa_sales_agent'] ) ) {
            $sales_agent_id = (int)$_REQUEST['wcb2bsa_sales_agent'];

            foreach ( $items as $user_id ) {
                if ( wcb2bsa_has_role( $user_id, 'customer' ) ) {
                    update_user_meta( $user_id, 'wcb2bsa_sales_agent', $sales_agent_id );
                }
                if ( wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
                    update_user_meta( $user_id, 'wcb2bsa_sales_agent_parent', $sales_agent_id );
                }
            }
        }
        if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
            if ( $action == 'wcb2bsa-change_status-action' && isset( $_REQUEST['wcb2bsa_status'] ) ) {
                $status = $_REQUEST['wcb2bsa_status'];

                // Validate request
                if ( ! is_numeric( $status ) ) { return $redirect_url; }

                foreach ( $items as $user_id ) {
                    // Skip if user hasn't a customer role
                    if ( wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
                        $current_status = get_the_author_meta( 'wcb2bsa_status', $user_id );

                        // Update status
                        update_user_meta( $user_id, 'wcb2bsa_status', intval( $status ) );

                        // If status is enabled, send approval confirmation to customer
                        if ( intval( $status ) && apply_filters( 'wcb2bsa_send_activation_notification', true ) ) {
                            if ( $current_status == 0 ) {
                                $emails = WC_Emails::instance()->emails;
                                if ( ! array_key_exists( 'WCB2BSA_Email_Sales_Agent_Status_Notification', $emails ) ) {
                                    $emails['WCB2BSA_Email_Sales_Agent_Status_Notification'] = include_once WCB2BSA_ABSPATH . 'includes/classes/emails/class-wcb2bsa-email-sales_agent-status-notification.php';
                                }
                                do_action( 'wcb2bsa_sales_agent_status_notification', $user_id );
                            }
                        }
                        // If status is disabled, logout from everywhere
                        if ( 0 == intval( $status ) ) {
                            $sessions = WP_Session_Tokens::get_instance( $user_id );
                            $sessions->destroy_all();
                        }
                    }
                }
            }
        }
        return $redirect_url;
    }

    /**
     * Add parameters to filter users
     *
     * @param WP_Query $query The WP_Query instance (passed by reference)
     */
    public function query_filters( $query ) {
        global $pagenow;

        if ( ! in_array( $query->get( 'role' ), get_option( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ) ) && 'users.php' == $pagenow && $query->get( 'limit' ) !== -1 ) {
            $meta_query = array();
            if ( isset( $_GET['wcb2bsa_sales_agent_search'] ) && $_GET['wcb2bsa_sales_agent_search'] !== '' ) {
                $meta_query[] = array(
                    'key'   => 'wcb2bsa_sales_agent',
                    'value' => (int)$_GET['wcb2bsa_sales_agent_search']
                );
            }
            if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
                if ( isset( $_GET['wcb2bsa_status_search'] ) && $_GET['wcb2bsa_status_search'] !== '' ) {
                    if ( $_GET['wcb2bsa_status_search'] == 0 ) {
                        $meta_query[] = array( array(
                            'relation' => 'OR',
                            array(
                                'key'   => 'wcb2bsa_status',
                                'value' => '',
                                'compare' => 'NOT EXISTS'
                            ),
                            array(
                                'key'   => 'wcb2bsa_status',
                                'value' => $_GET['wcb2bsa_status_search']
                            ) ) );
                    } else {
                        $meta_query[] = array(
                            'key'   => 'wcb2bsa_status',
                            'value' => $_GET['wcb2bsa_status_search']
                        );
                    }
                }
            }
            if ( ! empty( $meta_query ) ) {
                $query->set( 'meta_query', $meta_query );
            }
        }
    }

    /**
     * Add filters to users
     *
     * @param string $which The location of the extra table nav markup: 'top' or 'bottom
     */
    public function render_filters( $which ) {
        if ( $which != 'bottom' ) {
            $sales_agents = wcb2bsa_get_sales_agents();

            $filters = '<select name="wcb2bsa_sales_agent_search" id="wcb2bsa_sales_agent_search">';
            $filters .= '<option value="">' . esc_html__( 'Filter by sales agent', 'woocommerce-b2b-sales-agents' ) . '</option>';

            if ( ! empty( $sales_agents->get_results() ) ) {
                foreach ( $sales_agents->get_results() as $sales_agent ) {
                    $filters .= sprintf(
                        '<option value="%1$s" %2$s>%3$s</option>',
                        $sales_agent->ID,
                        ( isset( $_GET['wcb2bsa_sales_agent_search'] ) && $_GET['wcb2bsa_sales_agent_search'] != '' && (int)$_GET['wcb2bsa_sales_agent_search'] === $sales_agent->ID ? ' selected="selected"' : '' ),
                        $sales_agent->display_name
                    );
                }
            }
            $filters .= '</select>';
            wp_reset_postdata();

            if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
                $filters .= '<select name="wcb2bsa_status_search" id="wcb2bsa_status_search">';
                $filters .= '<option value="">' . esc_html__( 'Filter by status', 'woocommerce-b2b-sales-agents' ) . '</option>';

                $statuses = WCB2BSA_Configuration::get_sales_agent_statuses();

                foreach ( $statuses as $status => $status_label ) {
                    $filters .= sprintf(
                        '<option value="%1$s" %2$s>%3$s</option>',
                        $status,
                        ( isset( $_GET['wcb2bsa_status_search'] ) && $_GET['wcb2bsa_status_search'] != '' && intval( $_GET['wcb2bsa_status_search'] ) === $status ? ' selected="selected"' : '' ),
                        $status_label
                    );
                }
                $filters .= '</select>';
            }

            echo '<div class="alignleft actions">' . $filters . get_submit_button( esc_html__( 'Filter', 'woocommerce-b2b-sales-agents' ), '', 'filter', false ) . '</div>';
        }
    }

    /**
     * Add coupons restriction by sales agent
     *
     * @param integer $coupon_id Coupon ID
     * @param object $coupon Coupon object
     */
    public function coupon_restriction( $coupon_id, $coupon ) {
        include_once WCB2BSA_ABSPATH . 'includes/views/html-admin-coupons.php';
    }

    /**
     * Save coupons restriction by sales agent
     *
     * @param integer $post_id Coupon ID
     * @param object $coupon Coupon object
     */
    public function coupon_options_save( $post_id, $coupon ) {
        $value = isset( $_POST['_wcb2bsa_coupon_sales_agents'] ) ? $_POST['_wcb2bsa_coupon_sales_agents'] : false;
        update_post_meta( $post_id, '_wcb2bsa_coupon_sales_agents', $value );
    }

    /**
     * Allow shop manager to edit sales agent profiles
     *
     * @param array $roles Roles
     * @return array
     */
    public function shop_manager_editable_roles( $roles ) {
        $roles[] = 'sales_agent';
        return $roles;
    }

}

return new WCB2BSA_Admin_Users();