<?php

/**
 * WooCommerce B2B Sales Agents Configuration Class
 *
 * @version 1.3.1
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'WCB2BSA_Configuration', false ) ) :

    /**
     * WCB2BSA_Configuration Class
     */
    class WCB2BSA_Configuration {

        /**
         * Configuration
         *
         * @var array
         */
        private $configuration = array();

        /**
         * The single instance of the class
         *
         * @var WCB2BSA_Configuration
         */
        protected static $_instance = null;

        /**
         * Main WCB2BSA_Configuration Instance
         *
         * Ensures only one instance of WCB2BSA_Configuration is loaded or can be loaded
         * 
         * @static
         * @return WCB2BSA_Configuration - Main instance
         */
        public static function instance() {
            if ( is_null( self::$_instance ) ) {
                self::$_instance = new self();
            }
            return self::$_instance;
        }

        /**
         * Load WooCommerce B2B Sales Agents configuration
         */
        public static function get_sales_agent_statuses() {
            $sales_agent_statuses = apply_filters( 'wcb2bsa_sales_agent_statuses', array(
                0 => esc_html__( 'Inactive', 'woocommerce-b2b-sales-agents' ),
                1 => esc_html__( 'Active', 'woocommerce-b2b-sales-agents' )
            ) );
            return $sales_agent_statuses;
        }

        /**
         * Load WooCommerce B2B Sales Agents configuration
         */
        public static function get_order_item_types_eligible_for_commission( $only_keys = true ) {
            $order_item_types_eligible_for_commission = apply_filters( 'wcb2bsa_order_item_types_eligible_for_commission', array(
                'line_item' => esc_html__( 'Products', 'woocommerce-b2b-sales-agents' ),
                'fee'       => esc_html__( 'Fees', 'woocommerce-b2b-sales-agents' ),
                'shipping'  => esc_html__( 'Shipping costs', 'woocommerce-b2b-sales-agents' )
            ) );
            if ( $only_keys ) {
                $order_item_types_eligible_for_commission = array_keys( $order_item_types_eligible_for_commission );
            }
            return $order_item_types_eligible_for_commission;
        }

        /**
         * Get all possible order creators
         */
        public static function get_order_created_by() {
            return array(
                'admin'       => __( 'Admin', 'woocommerce-b2b-sales-agents' ),
                'customer'    => __( 'Customer', 'woocommerce-b2b-sales-agents' ),
                'sales_agent' => __( 'Sales agent', 'woocommerce-b2b-sales-agents' )
            );
        }

        /**
         * Retrieve all dashboard widgets and charts
         */
        public static function get_widgets( $sales_agent_id = false ) {
            $enabled_widgets = get_option( 'wcb2bsa_dashboard_widgets', array() );
            $enabled_charts = get_option( 'wcb2bsa_dashboard_charts', array() );

            $widgets = array(
                'sales_agent'   => get_userdata( $sales_agent_id ),
                'widgets'       => array(
                    'managed_subagents' => array(
                        'label' => esc_html__( 'Managed subagents number', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'managed_subagents', $enabled_widgets ) ? sprintf( '%s', count( wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id ) ) ) : false,
                        'icon'  => 'fas fa-user-tie'
                    ),
                    'managed_customers' => array(
                        'label' => esc_html__( 'Managed customers number', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'managed_customers', $enabled_widgets ) ? sprintf( '%s', wcb2bsa_get_customers_by_sales_agent( $sales_agent_id )->total_users ) : false,
                        'icon'  => 'fas fa-users'
                    ),
                    'total_orders' => array(
                        'label' => esc_html__( 'Total orders number', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'total_orders', $enabled_widgets ) ? sprintf( '%s', wcb2bsa_get_orders_by_sales_agent( $sales_agent_id )->total ) : false,
                        'icon'  => 'fas fa-shopping-cart'
                    ),
                    'total_amount' => array(
                        'label' => esc_html__( 'Total orders value', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'total_amount', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_orders_amount_by_sales_agent( $sales_agent_id ) ) ) : false,
                        'icon'  => 'fas fa-hand-holding-usd'
                    ),

                    'total_commissions' => array(
                        'label' => esc_html__( 'Your earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'total_commissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'total', false ) ) ) : false,
                        'icon'  => 'fas fa-coins'
                    ),
                    'paid_commissions' => array(
                        'label' => esc_html__( 'Your paid earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'paid_commissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'paid', false ) ) ) : false,
                        'icon'  => 'fas fa-wallet'
                    ),
                    'pending_commissions' => array(
                        'label' => esc_html__( 'Your pending earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'pending_commissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'pending', false ) ) ) : false,
                        'icon'  => 'fas fa-piggy-bank'
                    ),
                    'current_commission' => array(
                        'label' => esc_html__( 'Current commission percentage', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'current_commission', $enabled_widgets ) ? sprintf( '%s%%', get_the_author_meta( 'wcb2bsa_commission', $sales_agent_id ) ) : false,
                        'icon'  => 'fas fa-percentage'
                    ),

                    'total_subcommissions' => array(
                        'label' => esc_html__( 'Your subcommission earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'total_subcommissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'total', true ) ) ) : false,
                        'icon'  => 'fas fa-coins'
                    ),
                    'paid_subcommissions' => array(
                        'label' => esc_html__( 'Your paid subcommission earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'paid_subcommissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'paid', true ) ) ) : false,
                        'icon'  => 'fas fa-wallet'
                    ),
                    'pending_subcommissions' => array(
                        'label' => esc_html__( 'Your pending subcommission earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'pending_subcommissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'pending', true ) ) ) : false,
                        'icon'  => 'fas fa-piggy-bank'
                    ),
                    'current_subcommission' => array(
                        'label' => esc_html__( 'Current subcommission percentage', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'current_subcommission', $enabled_widgets ) ? sprintf( '%s%%', get_the_author_meta( 'wcb2bsa_subcommission', $sales_agent_id ) ) : false,
                        'icon'  => 'fas fa-percentage'
                    ),

                    'total_all_commissions' => array(
                        'label' => esc_html__( 'Total earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'total_all_commissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'total' ) ) ) : false,
                        'icon'  => 'fas fa-coins'
                    ),
                    'paid_all_commissions' => array(
                        'label' => esc_html__( 'Total paid earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'paid_all_commissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'paid' ) ) ) : false,
                        'icon'  => 'fas fa-wallet'
                    ),
                    'pending_all_commissions' => array(
                        'label' => esc_html__( 'Total pending earnings', 'woocommerce-b2b-sales-agents' ),
                        'value' => $sales_agent_id && array_key_exists( 'pending_all_commissions', $enabled_widgets ) ? sprintf( '%s', wc_price( wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, 'pending' ) ) ) : false,
                        'icon'  => 'fas fa-piggy-bank'
                    )
                ),
                'stats'         => array(
                    'subagents'     => $sales_agent_id && array_key_exists( 'subagents', $enabled_charts ) ? wcb2bsa_get_stats( $sales_agent_id, 'bar', 'subagents' ) : false,
                    'customers'     => $sales_agent_id && array_key_exists( 'customers', $enabled_charts ) ? wcb2bsa_get_stats( $sales_agent_id, 'bar', 'customers' ) : false,
                    'commissions'   => $sales_agent_id && array_key_exists( 'commissions', $enabled_charts ) ? wcb2bsa_get_stats( $sales_agent_id, 'bar', 'commissions' ) : false,
                    'orders'        => $sales_agent_id && array_key_exists( 'orders', $enabled_charts ) ? wcb2bsa_get_stats( $sales_agent_id, 'bar', 'orders' ) : false
                )
            );

            $widgets = apply_filters( 'wcb2bsa_dashboard_widgets', $widgets );
            if ( $sales_agent_id ) {
                $widgets['widgets'] = array_intersect_key( $widgets['widgets'], $enabled_widgets );
                $widgets['stats'] = array_intersect_key( $widgets['stats'], $enabled_charts );
            }
            return $widgets;
        }

        /**
         * Load WooCommerce B2B Sales Agents configuration
         */
        public static function get_sales_agents_statuses() {
            $sales_agents_statuses = apply_filters( 'wcb2bsa_sales_agents_statuses', array(
                0 => esc_html__( 'Inactive', 'woocommerce-b2b-sales-agents' ),
                1 => esc_html__( 'Active', 'woocommerce-b2b-sales-agents' )
            ) );
            return $sales_agents_statuses;
        }

    }

endif;