<?php

/**
 * WooCommerce B2B Sales Agents Frontend Checkout set-up Class
 *
 * @version 1.2.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Frontend_Checkout Class
 */
class WCB2BSA_Frontend_Checkout {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        if ( get_option( 'wcb2bsa_default_sales_agent' ) === 'choose' ) {
            add_filter( apply_filters( 'wcb2bsa_sales_agent_checkout_position', 'woocommerce_after_checkout_billing_form' ), array( $this, 'sales_agent_add_field' ) );
            add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'sales_agent_save_field' ) );
        }
        if ( get_option( 'wcb2bsa_edit_customer_data' ) != 'yes' ) {
            add_action( 'init', array( $this, 'disable_customer_edit_page' ) );
            add_filter( 'woocommerce_billing_fields', array( $this, 'disable_customer_edit' ), 11 );
            add_filter( 'woocommerce_shipping_fields', array( $this, 'disable_customer_edit' ), 11 );
            add_action( 'woocommerce_checkout_before_customer_details', array( $this, 'disable_customer_edit_message' ) );
            add_action( 'woocommerce_account_edit-account_endpoint', array( $this, 'disable_customer_edit_message' ) );
            add_action( 'woocommerce_before_edit_address_form_billing', array( $this, 'disable_customer_edit_message' ) );
            add_filter( 'woocommerce_account_menu_items', array( $this, 'hide_edit_account_page' ) );
        }
        if ( get_option( 'wcb2bsa_edit_prices' ) == 'yes' ) {
            add_filter( 'woocommerce_cart_item_price', array( $this, 'edit_prices' ), 99, 3 );
            add_filter( 'woocommerce_product_get_price', array( $this, 'display_edited_price' ), 99, 2 );
            add_filter( 'woocommerce_product_variation_get_price', array( $this, 'display_edited_price' ), 99, 2 );
            add_filter( 'woocommerce_update_cart_action_cart_updated', array( $this, 'update_edited_price' ) );
            add_action ('woocommerce_new_order_item', array( $this, 'save_edited_price' ), 10, 2 );
        }
        add_filter( 'woocommerce_coupon_is_valid', array( $this, 'coupon_is_valid' ), 10, 3 );
    }

    /**
     * Add sales agent field to customer (in checkout) when "Allow to chose" enabled
     * 
     * @param object $checkout Checkout instance
     */
    public function sales_agent_add_field( $checkout ) {
        $sales_agent_id = false;
        if ( is_user_logged_in() ) {
            $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', get_current_user_id() );
        }

        if ( empty( $sales_agent_id ) ) {
            $sales_agents = wcb2bsa_get_sales_agents();
            if ( $sales_agents->get_total() ) {
                woocommerce_form_field( 'wcb2bsa_sales_agent', array(
                    'type'          => 'select',
                    'class'         => array( 'form-row-wide' ),
                    'label'         => esc_html__( 'Choose your sales agent', 'woocommerce-b2b-sales-agents' ),
                    'required'      => true,
                    'options'       => array_reduce( $sales_agents->get_results(), function( $result, $item ) {
                        $item = (array)$item->data;
                        $result[$item['ID']] = $item['display_name'];
                        return $result;
                    } )
                ), $checkout->get_value( 'wcb2bsa_sales_agent' ) );
            }
        }
    }

    /**
     * Save sales agent field to customer (in checkout) when "Allow to chose" enabled
     * 
     * @param integer $order_id Current order ID
     */
    public function sales_agent_save_field( $order_id ) {
        if ( ! empty( $_POST['wcb2bsa_sales_agent'] ) ) {
            $order = new WC_Order( $order_id );
            update_user_meta( $order->get_customer_id(), 'wcb2bsa_sales_agent', (int)$_POST['wcb2bsa_sales_agent'] );
            // Check for guest customers
            if ( empty( $order->get_customer_id() ) ) {
                update_post_meta( $order_id, 'wcb2bsa_sales_agent', (int)$_POST['wcb2bsa_sales_agent'] );
            }
        }
    }

    /**
     * Remove default content from edit account page
     */
    public function disable_customer_edit_page() {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                remove_action( 'woocommerce_account_edit-account_endpoint', 'woocommerce_account_edit_account' );
            }
        }
    }

    /**
     * Display a message in checkout to notice readonly fields
     */
    public function disable_customer_edit_message() {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                printf( '<div class="woocommerce-message wcb2bsa_checkout_notice">%s</div>', __( 'You are not allowed to change customer data, you can only enter missing data.', 'woocommerce-b2b-sales-agents' ) );
            }
        }
    }

    /**
     * Prevent address edit in checkout
     * 
     * @param  array $fields Checkout billing/shipping fields
     * @return array
     */
    public function disable_customer_edit( $fields ) {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                foreach ( $fields as $key => $field ) {
                    if ( true == $field['required'] && empty( get_user_meta( get_current_user_id(), $key, true ) ) ) { continue; }
                    $fields[$key]['class'] = array('wcb2bsa-readonly');
                    $fields[$key]['custom_attributes'] = array(
                        'readonly' => 'readonly'
                    );
                }
            }
        }
        return $fields;
    }

    /**
     * Hide edit account page
     * 
     * @param array $items Navigation menu items
     * @return array
     */
    public function hide_edit_account_page( $items ) {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                unset( $items['edit-account'] );
            }
        }
        return $items;
    }

    /**
     * Transform price field in cart into input field to allow to change price
     *
     * @param integer $price Current cart item price
     * @param object $cart_item Current cart item object
     * @param string $cart_item_key Current cart item object hash
     * @return string
     */
    public function edit_prices( $price, $cart_item, $cart_item_key ) {
        // Check if is a sales agent switched account
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                $min_price = get_post_meta( $cart_item['product_id'], 'wcb2bsa_min_price', true );
                if ( $cart_item['variation_id'] ) {
                    $min_price = get_post_meta( $cart_item['variation_id'], 'wcb2bsa_min_price', true );
                }
                $attributes = apply_filters( 'wcb2bsa_cart_input', array(
                    'class' => 'input-text text',
                    'min'   => $min_price ?: 0,
                    'step'  => '0.01'
                ) );
                $field = sprintf( '<input type="number" name="cart[%s][wcb2bsa_custom_price]" value="%s" %s>',
                    $cart_item_key,
                    $cart_item['data']->get_price(),
                    implode( '', array_map( function( $key, $value ) {
                        return sprintf( ' %s="%s"', $key, $value );
                    }, array_keys( $attributes ), $attributes ) )
                );
                return $field;
            }
        }
        return $price;
    }

    /**
     * Prevent edited price in cart is overwritten
     * 
     * @param  float $price Current product price
     * @param  object $object Current product or variation instance
     * @return float
     */
    public function display_edited_price( $price, $object ) {
        if ( defined( 'REST_REQUEST' ) ) { return $price; }

        $key = 'product_id';
        if ( 'variation' === $object->get_type() ) {
            $key = 'variation_id';
        }

        if ( is_array( WC()->cart->get_cart() ) ) {
            foreach ( WC()->cart->get_cart() as $cart_item ) {
                if ( $cart_item[$key] == $object->get_id() ) {
                    if ( isset( $cart_item['wcb2bsa_custom_price'] ) ) {
                        return $cart_item['wcb2bsa_custom_price'];
                    }
                }
            }
        }
        return $price;
    }

    /**
     * Alter cart item price
     *
     * @param boolean $cart_updated Is cart already updated?
     */
    public function update_edited_price( $cart_updated ) {
        // Skip if is admin
        if ( is_user_logged_in() && ! is_admin() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                $cart_totals = isset( $_POST['cart'] ) ? $_POST['cart'] : array();
                if ( ! WC()->cart->is_empty() ) {
                    // Cicle products in cart
                    foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
                        // Retrive custom posted price
                        $custom_item_price = $cart_item['data']->get_price();
                        if ( isset( $cart_item['wcb2bsa_custom_price'] ) ) {
                            $custom_item_price = $cart_item['wcb2bsa_custom_price'];
                        }
                        if ( isset( $cart_totals[ $cart_item_key ]['wcb2bsa_custom_price'] ) ) {
                            $custom_item_price = $cart_totals[ $cart_item_key ]['wcb2bsa_custom_price'];
                        }
                        // Check min price
                        $min_price = get_post_meta( $cart_item['product_id'], 'wcb2bsa_min_price', true );
                        if ( $cart_item['variation_id'] ) {
                            $min_price = get_post_meta( $cart_item['variation_id'], 'wcb2bsa_min_price', true );
                        }
                        if ( (float)$min_price > $custom_item_price ) {
                            wc_add_notice( sprintf( __( 'The minimum price for the product "%s" is %s.', 'woocommerce-b2b-sales-agents' ), $cart_item['data']->get_name(), wc_price( $min_price ) ), 'error' );
                            continue;
                        }
                        // Set custom price
                        $cart_item['wcb2bsa_custom_price'] = $custom_item_price;
                        $cart_item['data']->set_price( $custom_item_price );

                        WC()->cart->cart_contents[$cart_item_key] = $cart_item;
                    }
                    if ( ! empty( $_POST['update_cart'] ) ) {
                        $cart_updated = true;
                        WC()->cart->set_session();
                        wc_add_notice( __( 'Cart updated.', 'woocommerce-b2b-sales-agents' ), 'success' );
                    }
                }
            }
        }
        return $cart_updated;
    }

    /**
     * Save as item meta if sales agent changed price
     *
     * @param integer $item_id Current item ID
     * @param array $values Current item data
     */
    public function save_edited_price( $item_id, $values ) {
        // Check if is a sales agent switched account
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                // Check if custom price is set and save meta
                if ( isset( $values->legacy_values['wcb2bsa_custom_price'] ) || isset( $values['wcb2bsa_custom_price'] ) ) {
                    wc_add_order_item_meta( $item_id, 'wcb2bsa_custom_price', true );
                }
            }
        }
    }

    /**
     * Check coupons restriction by group
     *
     * @param boolean $is_valid Check if coupon is valid
     * @param object $coupon Coupon object
     * @param mixed $discount Discount
     */
    public function coupon_is_valid( $is_valid, $coupon, $discount ) {
        // Skip if admin
        if ( is_admin() ) { return $is_valid; }
        // Get allowed groups
        $sales_agents = $coupon->get_meta( '_wcb2bsa_coupon_sales_agents' );
        if ( empty( $sales_agents ) ) { return $is_valid; }

        // Get current user ID
        $user_id = get_current_user_id();
        // Get sales agent ID
        if ( $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $user_id ) ) {
            if ( in_array( $sales_agent_id, $sales_agents ) ) {
                return $is_valid;
            }
        }        
        return false;
    }

}

return new WCB2BSA_Frontend_Checkout();