<?php

/**
 * WooCommerce B2B Sales Agents Frontend Orders set-up Class
 *
 * @version 1.2.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Frontend_Orders Class
 */
class WCB2BSA_Frontend_Orders {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_action( 'woocommerce_new_order', array( $this, 'save_order_meta' ), 10, 1 );
        add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_order_commission_amount' ), 10, 2 );
        add_filter( 'woocommerce_email_recipient_new_order', array( $this, 'notify' ), 10, 2 );
        add_filter( 'user_has_cap', array( $this, 'allow_order_again' ), 10, 3 );
    }

    /**
     * On new order, assign order meta
     *
     * @param int $order_id The processing order ID
     */
    public function save_order_meta( $order_id ) {
        $order = new WC_Order( $order_id );

        $sales_agent_id = 0;
        if ( ! empty( $_POST['wcb2bsa_sales_agent'] ) ) {
            $sales_agent_id = (int)$_POST['wcb2bsa_sales_agent'];
        } elseif ( ! empty( $order->get_customer_id() ) ) {
            $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $order->get_customer_id() );
        }

        update_post_meta( $order_id, '_wcb2bsa_order_version', str_replace( '.', '', WCB2BSA()->version ) );
        update_post_meta( $order_id, 'wcb2bsa_sales_agent', $sales_agent_id );
        update_post_meta( $order_id, 'wcb2bsa_order_item_types_eligible_for_commission', get_option( 'wcb2bsa_order_item_types_eligible_for_commission' ) );
        if ( get_transient( 'wcb2bsa_switched_' . $order->get_customer_id() ) ) {
            update_post_meta( $order_id, 'wcb2bsa_created_by', 'sales_agent' );
        } else {
            update_post_meta( $order_id, 'wcb2bsa_created_by', 'customer' );
        }
    }

    /**
     * On new order, calculate total commission amount and save
     *
     * @param int $order_id The processing order ID
     * @param array $data The processing order data
     */
    public function save_order_commission_amount( $order_id, $data ) {
        // Save order total commission amount
        update_post_meta( $order_id, 'wcb2bsa_commission_amount', wcb2bsa_order_commission_amount_evaluate( $order_id ) );
    }

    /**
     * Allow Sales agent to order again
     *
     *   @param array $allcaps All sales agent capabilities
     *   @param array $caps Capabilities
     *   @param args $args Arguments
     */
    public function allow_order_again( $allcaps, $caps, $args ) {
        if ( isset( $caps[0] ) && $caps[0] == 'order_again' ) {
            $sales_agent_id = intval( $args[1] );
            $order          = wc_get_order( $args[2] );
            $customer_id    = $order->get_customer_id();

            if ( ! wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) { return $allcaps; }

            // Check if current customer sales agent is the same that are ordering again
            if ( $can_order = wcb2bsa_can_write( $sales_agent_id, $customer_id ) ) {
                $allcaps['order_again'] = true;

                set_transient( 'wcb2bsa_switched_' . $customer_id, $sales_agent_id, 86400 );
                setcookie( 'wcb2bsa_switched', 'true', 0, '/', COOKIE_DOMAIN, is_ssl(), true );

                wp_set_current_user( $customer_id );
                wp_set_auth_cookie( $customer_id );
            }
            
        }
        return $allcaps;
    }

    /**
     * Add another email recipient for admin New Order emails
     *
     * @param string $recipient Comma-separated string of email recipients (will turn into an array after this filter!)
     * @param WC_Order $order Order object for which the email is sent
     * @return string
     */
    public function notify( $recipient, $order ) {
        $page = $_GET['page'] = isset( $_GET['page'] ) ? $_GET['page'] : '';
        if ( 'wc-settings' === $page ) {
            return $recipient; 
        }
        if ( ! $order instanceof WC_Order ) {
            return $recipient; 
        }

        if ( $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent' ) ) {
            $sales_agent = get_userdata( $sales_agent_id );
            if ( false !== $sales_agent && wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) {
                $notify = get_the_author_meta( 'wcb2bsa_order_notification', $sales_agent_id );
                if ( 1 == $notify ) {
                    $recipient .= sprintf( ', %s', $sales_agent->user_email );
                }
            }
        }
        return $recipient;
    }

}

return new WCB2BSA_Frontend_Orders();