<?php

/**
 * WooCommerce B2B Sales Agents Install Class
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Install Class
 */
class WCB2BSA_Install {

    /**
     * DB updates and callbacks that need to be run per version
     *
     * @var array
     */
    private static $db_updates = array(
        '1.0.5' => array(
            'wcb2bsa_order_created_by_105'
        ),
        '1.0.9' => array(
            'wcb2bsa_dashboard_page_109',
            'wcb2bsa_edit_customer_data_109'
        ),
        '1.1.1' => array(
            'wcb2bsa_add_capabilities_111'
        ),
        '1.1.2' => array(
            'wcb2bsa_order_total_commission_112',
            'wcb2bsa_earning_endpoint_112'
        ),
        '1.2.0' => array(
            'wcb2bsa_order_item_commission_amount_120',
            'wcb2bsa_order_commission_payment_120'
        ),
        '1.3.2' => array(
            'wcb2bsa_set_suitable_roles_132'
        )
    );

    /**
     * Init install class
     */
    public static function init() {
        // Process queue
        add_action( 'admin_init', array( __CLASS__, 'check_version' ), 5 );
        add_action( 'wcb2bsa_run_update_callback', array( __CLASS__, 'run_update_callback' ) );
    }

    /**
     * Check if database need updates
     */
    public static function check_version() {
        $current_version = get_option( 'wcb2bsa_version', null );
        if ( is_null( $current_version ) ) {
            self::install();
        } else {
            if ( self::needs_db_update() ) {
                if ( isset( $_GET['do_update_wcb2bsa'] ) && ! empty( $_GET['do_update_wcb2bsa'] ) ) {
                    if ( check_admin_referer( 'wcb2bsa_db_update', 'wcb2bsa_db_update_nonce' ) ) {
                        set_transient( 'wcb2bsa_update_scheduled', 'yes', MINUTE_IN_SECONDS * 10 );
                        self::update();
                        wp_redirect( admin_url( 'tools.php?page=action-scheduler' ) );
                        exit;
                    }
                } else {
                    if ( false === get_transient( 'wcb2bsa_update_scheduled' ) ) {
                        add_action( 'admin_notices', array( get_called_class(), 'update_notice' ) );
                    } else {
                        $next_scheduled_date = WC()->queue()->get_next( 'wcb2bsa_run_update_callback', null, 'wcb2bsa-db-updates' );
                        if ( $next_scheduled_date ) {
                            add_action( 'admin_notices', array( get_called_class(), 'updating_notice' ) );
                        } else {
                            self::update_version();
                            delete_transient( 'wcb2bsa_update_scheduled' );
                            add_action( 'admin_notices', array( get_called_class(), 'updated_notice' ) );
                        }
                    }
                }
            } else {
                if ( version_compare( get_option( 'wcb2bsa_version', 0 ), WCB2BSA()->version, '<' ) ) {
                    self::update_version();
                }
            }
        }
    }

    /**
     * Install WCB2BSA
     */
    public static function install() {
        if ( 'yes' === get_transient( 'wcb2bsa_installing' ) ) {
            return;
        }

        set_transient( 'wcb2bsa_installing', 'yes', MINUTE_IN_SECONDS * 10 );

        self::create_roles();
        self::set_suitable_roles();
        self::create_options();
        self::create_dashboard_page();
        self::update_version();

        flush_rewrite_rules();

        delete_transient( 'wcb2bsa_installing' );
    }

    /**
     * Update WCB2BSA
     */
    public static function update() {
        if ( 'yes' === get_transient( 'wcb2bsa_updating' ) ) {
            return;
        }

        set_transient( 'wcb2bsa_updating', 'yes', MINUTE_IN_SECONDS * 10 );

        self::schedule_update_callbacks();

        delete_transient( 'wcb2bsa_updating' );
    }

    /**
     * Is a DB update needed?
     */
    private static function needs_db_update() {
        $current_version = get_option( 'wcb2bsa_version', null );
        $updates         = self::$db_updates;
        $update_versions = array_keys( $updates );
        usort( $update_versions, 'version_compare' );
        return ! is_null( $current_version ) && version_compare( $current_version, end( $update_versions ), '<' );
    }

    /**
     * Schedule database updates
     */
    private static function schedule_update_callbacks() {
        $current_version = get_option( 'wcb2bsa_version' );
        $loop            = 0;
        foreach ( self::$db_updates as $version => $update_callbacks ) {
            if ( version_compare( $current_version, $version, '<' ) ) {
                foreach ( $update_callbacks as $update_callback ) {
                    // Add to scheduled queue
                    WC()->queue()->schedule_single(
                        time() + $loop,
                        'wcb2bsa_run_update_callback',
                        array(
                            'update_callback' => $update_callback,
                        ),
                        'wcb2bsa-db-updates'
                    );
                    $loop++;
                }
            }
        }
    }

    /**
     * Triggered when a callback will run
     *
     * @param string $callback Callback name
     */
    public static function run_update_callback( $update_callback ) {
        include_once WCB2BSA_ABSPATH . 'includes/wcb2bsa-update-functions.php';

        if ( is_callable( $update_callback ) ) {
            // Logger
            wc_get_logger()->debug( sprintf( '::%s:: callback started', $update_callback ), array(
                'source' => 'wcb2bsa'
            ) );

            $result = (bool) call_user_func( $update_callback );
            self::run_update_callback_end( $update_callback, $result );

            // Logger
            wc_get_logger()->debug( sprintf( '::%s:: callback done [%s]', $update_callback, $result ? 'OK' : 'KO' ), array(
                'source' => 'wcb2bsa'
            ) );
        }
    }

    /**
     * Triggered when a callback has ran
     *
     * @param string $callback Callback name
     * @param bool   $result Return value from callback. False need to run again
     */
    protected static function run_update_callback_end( $callback, $result ) {
        if ( true === $result ) {
            return;
        }
        
        // If callback return false, needs to run again
        WC()->queue()->add(
            'wcb2bsa_run_update_callback',
            array(
                'update_callback' => $callback,
            ),
            'wcb2bsa-db-updates'
        );
    }

    /**
     * Create roles and capabilities
     */
    private static function create_roles() {
        add_role( 'sales_agent', esc_html__( 'Sales agent', 'woocommerce-b2b-sales-agents' ), array(
            'read' => true,
            'edit_shop_orders' => true
        ) );
    }

    /**
     * Set roles suitable as customer/sales agent
     *
     * @return boolean
     */
    private static function set_suitable_roles() {
        if ( false === get_option( 'wcb2bsa_has_role_customer', false ) ) {
            update_option( 'wcb2bsa_has_role_customer', apply_filters( 'wcb2bsa_has_role_customer', array( 'customer' ) ) );
        }
        if ( false === get_option( 'wcb2bsa_has_role_sales_agent', false ) ) {
            update_option( 'wcb2bsa_has_role_sales_agent', apply_filters( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ) );
        }
        return true;
    }

    /**
     * Create options
     * - Widgets
     * - Charts
     */
    private static function create_options() {
        $widgets = WCB2BSA_Configuration::get_widgets();
        add_option( 'wcb2bsa_dashboard_widgets', array_fill_keys( array_keys( $widgets['widgets'] ), 1 ) );
        add_option( 'wcb2bsa_dashboard_charts', array_fill_keys( array_keys( $widgets['stats'] ), 1 ) );
    }

    /**
     * Create sales agents dashboard page
     */
    private static function create_dashboard_page() {
        if ( get_option( 'wcb2bsa_dashboard_page', false ) ) { return true; }

        // Insert quick order page
        $dashboard_page = wp_insert_post( array(
            'post_title'   => esc_html__( 'Sales agents dashboard', 'woocommerce-b2b-sales-agents' ),
            'post_type'    => 'page',
            'post_status'  => 'draft',
            'post_content' => '[woocommerce_my_account]'
        ) );

        return update_option( 'wcb2bsa_dashboard_page', $dashboard_page );
    }

    /**
     * Update WCB2BSA version to current
     */
    private static function update_version() {
        $prev_version = get_option( 'wcb2bsa_version', 0 );
        $next_version = WCB2BSA()->version;
        if ( $prev_version !== $next_version ) {
            // Logger
            wc_get_logger()->debug( sprintf( '%s -> %s', $prev_version, $next_version), array(
                'source' => 'wcb2bsa'
            ) );

            update_option( 'wcb2bsa_version', $next_version );
        }
    }

    /**
     * Display update needs notice
     */
    public static function update_notice() {
        include_once WCB2BSA_ABSPATH . 'includes/views/notices/html-admin-notice-update.php';
    }

    /**
     * Display update processing notice
     */
    public static function updating_notice() {
        include_once WCB2BSA_ABSPATH . 'includes/views/notices/html-admin-notice-updating.php';
    }

    /**
     * Display update complete needs notice
     */
    public static function updated_notice() {
        include_once WCB2BSA_ABSPATH . 'includes/views/notices/html-admin-notice-updated.php';
    }

}