<?php

/**
 * WooCommerce B2B Sales Agents Waiting payment Gateway
 *
 * @version 1.1.2
 */

defined( 'ABSPATH' ) || exit;

class WCB2BSA_Gateway_Waiting extends WC_Payment_Gateway {

    /**
     * Constructor for the gateway.
     */
    public function __construct() {
  
        $this->id                 = 'wcb2bsa_waiting_gateway';
        $this->icon               = apply_filters( 'wcb2bsa_waiting_gateway_icon', '' );
        $this->has_fields         = false;
        $this->method_title       = __( 'Waiting payment', 'woocommerce-b2b-sales-agents' );
        $this->method_description = __( 'Allows sales agents to place "waiting payment" orders, that customers will pay later.', 'woocommerce-b2b-sales-agents' );
      
        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();
      
        // Define user set variables
        $this->title        = $this->get_option( 'title' );
        $this->description  = $this->get_option( 'description' );
        $this->sales_agent_instructions = $this->get_option( 'sales_agent_instructions', $this->description );
        $this->customer_instructions = $this->get_option( 'customer_instructions', $this->description );
      
        // Actions
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'thankyou_page' ) );
      
        // Customer Emails
        add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 );
    }

    /**
     * Initialize Gateway Settings Form Fields
     */
    public function init_form_fields() {
        $this->form_fields = apply_filters( 'wc_offline_form_fields', array(
            'enabled' => array(
                'title'   => __( 'Enable/Disable', 'woocommerce-b2b-sales-agents' ),
                'type'    => 'checkbox',
                'label'   => __( 'Enable orders with waiting payment for sales agents', 'woocommerce-b2b-sales-agents' ),
                'default' => 'yes'
            ),
            'title' => array(
                'title'       => __( 'Title', 'woocommerce-b2b-sales-agents' ),
                'type'        => 'text',
                'description' => __( 'This controls the title for the payment method the customer sees during checkout.', 'woocommerce-b2b-sales-agents' ),
                'default'     => __( 'Waiting payment', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'    => true,
            ),
            'description' => array(
                'title'       => __( 'Description', 'woocommerce-b2b-sales-agents' ),
                'type'        => 'textarea',
                'description' => __( 'Payment method description that the customer will see on your checkout.', 'woocommerce-b2b-sales-agents' ),
                'default'     => __( 'The customer will receive an email with order details and a payment link.', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'    => true,
            ),
            'sales_agent_instructions' => array(
                'title'       => __( 'Thank you page instructions', 'woocommerce-b2b-sales-agents' ),
                'type'        => 'textarea',
                'description' => __( 'Instructions that will be added to the thank you page.', 'woocommerce-b2b-sales-agents' ),
                'default'     => __( 'You placed an order on your customer behalf with a waiting payment. The order is now awaiting for customer payment.', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'    => true,
            ),
            'customer_instructions' => array(
                'title'       => __( 'Customer email instructions', 'woocommerce-b2b-sales-agents' ),
                'type'        => 'textarea',
                'description' => __( 'Instructions that will be added to customer emails.', 'woocommerce-b2b-sales-agents' ),
                'default'     => __( 'Your sales agent has placed an order on your behalf with a waiting payment. The order is now awaiting for your payment.', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'    => true,
            ),
        ) );
    }

    /**
     * Output for the order received page.
     */
    public function thankyou_page() {
        if ( $this->sales_agent_instructions ) {
            echo wpautop( wptexturize( $this->sales_agent_instructions ) );
        }
    }

    /**
     * Add content to the WC emails.
     *
     * @access public
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
    public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
        if ( $this->customer_instructions && ! $sent_to_admin && $this->id === $order->get_payment_method() && $order->has_status( 'waiting' ) ) {
            $checkout_url = ' <a href="' . $order->get_checkout_payment_url() . '">' . __( 'Go to payment page', 'woocommerce-b2b-sales-agents' ) . '</a>';

            echo wpautop( wptexturize( $this->customer_instructions . $checkout_url ) ) . PHP_EOL;
        }
    }

    /**
     * Process the payment and return the result
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment( $order_id ) {
        $order = wc_get_order( $order_id );
        // Mark as on-hold (we're awaiting the payment)
        $order->update_status( 'waiting' );
        
        // Remove cart
        WC()->cart->empty_cart();
        
        // Return thankyou redirect
        return array(
            'result'    => 'success',
            'redirect'  => $this->get_return_url( $order )
        );
    }

}